-- LinkWrangler-Auctioneer patch plugin
-- by brykrys
-- version 1.69
-- release

-- Depends on the TOC to force various other AddOns to load first (if they are installed and enabled)

local function InstallLext()
	if LibStub then
		local lext = LibStub ("LibExtraTip-1", true) -- silent
		if lext then
			LinkWrangler.RegisterCallback ("LWAuctioneer", function (frame)
				lext:RegisterTooltip(frame) -- Register LinkWrangler tooltips with Auctioneer's LibExtraTip library
			end, "allocate")
			InstallLext = nil
			return true
		end
	end
end

InstallLext()

-- All further code either uses Stubby directly, or only applies to AddOns which depend on Stubby themselves
if not Stubby then
	return
end

-- Helper function to test if AddOn is enabled and loadable
local TestAddOn = function(addon)
	local _, _, _, loadable, reason = GetAddOnInfo(addon)
	if not loadable then
		loadable = reason == "DEMAND_LOADED" or reason == "DEP_DEMAND_LOADED"
	end
	local enabled = GetAddOnEnableState(player, addon)
	enabled = enabled and enabled > 0
	return enabled and loadable
end

-- LoadOnDemand AddOns may not have loaded yet - we have to detect them individually as they load
if InstallLext then
	local flagEnx, flagInf
	flagEnx = TestAddOn("Enchantrix")
	if flagEnx then
		Stubby.RegisterAddOnHook("Enchantrix", "LWAuctioneer", function()
			if Enchantrix then -- check it has loaded
				Stubby.UnregisterAddOnHook("Enchantrix", "LWAuctioneer")
				if not InstallLext or InstallLext() then
					if flagInf then
						Stubby.UnregisterAddOnHook("Informant", "LWAuctioneer")
					end
				end
			end
		end)
	end
	flagInf = TestAddOn("Informant")
	if flagInf then
		Stubby.RegisterAddOnHook("Informant", "LWAuctioneer", function()
			if Informant then -- check it has loaded
				Stubby.UnregisterAddOnHook("Informant", "LWAuctioneer")
				if not InstallLext or InstallLext() then
					if flagEnx then
						Stubby.UnregisterAddOnHook("Enchantrix", "LWAuctioneer")
					end
				end
			end
		end)
	end
end

-- Install extra hooks for specific Auc-Advanced modules:
-- If those modules are disembedded they might not be loaded yet, in which case use Stubby to detect when they load
if AucAdvanced then
	local AppraiserButtonHook

	if LinkWrangler.ButtonClick then -- check that the installed version of LinkWrangler supports button hooking
		local SnatchTest, AppraiserTest
		local oldButtonClick = LinkWrangler.ButtonClick
		-- the hook
		LinkWrangler.ButtonClick = function(button, ...)
			if button.ButtonType == "Relink" or button.ButtonType == "Link" then
				if SnatchTest and SnatchTest(button) then return end
				if AppraiserTest and AppraiserTest(button) then return end
			end
			return oldButtonClick(button, ...)
		end

		-- Make SnatchTest
		local function SnatchButtonHook(event, gui)
			if event == "guiconfig" then
				AucSearchUI.RemoveCallback("LWAuctioneer", SnatchButtonHook)
				local Snatch = AucSearchUI.Searchers.Snatch
				if not Snatch then return end
				local SearchGUI = gui
				-- We use SearchUI's gui to detect when the Snatch tab is visible
				SnatchTest = function(button)
					if SearchGUI:IsShown() and SearchGUI.config.selectedTab == Snatch.tabname then
						local _, link = button:GetParent():GetItem()
						if link then
							Snatch.SetWorkingItem(link)
							return true -- chain stops here
						end
					end
				end
			end
		end
		if AucSearchUI then -- already loaded
			AucSearchUI.AddCallback("LWAuctioneer", SnatchButtonHook)
		else
			if TestAddOn("Auc-Util-SearchUI") then
				Stubby.RegisterAddOnHook("Auc-Util-SearchUI", "LWAuctioneer", function()
					if AucSearchUI then
						Stubby.UnregisterAddOnHook("Auc-Util-SearchUI", "LWAuctioneer")
						AucSearchUI.AddCallback("LWAuctioneer", SnatchButtonHook)
					end
				end)
			end
		end

		-- Make AppraiserTest
		AppraiserButtonHook = function ()
			AppraiserTest = function(button)
				local frame = AucAdvAppraiserFrame
				if frame and frame.salebox and frame.salebox:IsVisible() then
					local _, link = button:GetParent():GetItem()
					if link then
						frame.GetItemByLink(link)
						return true -- break the chain
					end
				end
			end
		end
	end

	if AucAdvanced.Modules.Util.Appraiser then -- already loaded
		if AppraiserButtonHook then AppraiserButtonHook() end
	else -- hook Stubby to detect when it loads
		if TestAddOn("Auc-Util-Appraiser") then
			Stubby.RegisterAddOnHook("Auc-Util-Appraiser", "LWAuctioneer", function()
				if AucAdvanced.Modules.Util.Appraiser then
					Stubby.UnregisterAddOnHook("Auc-Util-Appraiser", "LWAuctioneer")
					if AppraiserButtonHook then AppraiserButtonHook() end
				end
			end)
		end
	end

end
